package org.jeecg.modules.bytedance.advertise.service.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.lang3.StringUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.modules.bytedance.advertise.entity.ByteDanceGeneralCopywriter;
import org.jeecg.modules.bytedance.advertise.mapper.ByteDanceGeneralCopywriterMapper;
import org.jeecg.modules.bytedance.advertise.mapper.BytedanceVideoSlogenInfoMapper;
import org.jeecg.modules.bytedance.advertise.service.IByteDanceGeneralCopywriterService;
import org.jeecg.modules.bytedance.common.utils.Check;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;
import java.util.Map;

/**
 * 通用文案库
 * @author jeecg-boot
 * 2021-05-28
 * @version V1.0
 */
@Service
public class ByteDanceGeneralCopywriterServiceImpl extends ServiceImpl<ByteDanceGeneralCopywriterMapper, ByteDanceGeneralCopywriter> implements IByteDanceGeneralCopywriterService {
    @Resource
    ByteDanceGeneralCopywriterMapper byteDanceGeneralCopywriterMapper;


    @Override
    public Result getGeneralCopywriterReport(String accountId, String keyWord, String startTime, String endTime, Integer pageNum, Integer pageSize){
        PageHelper.startPage(pageNum,pageSize);
        List<Map<String, Object>> generalCopywriteReportList = byteDanceGeneralCopywriterMapper.selectGeneralCopywriterReport(accountId,keyWord, startTime, endTime);
        if(Check.isNull(generalCopywriteReportList)){
            return Result.error("还没有此文案报表数据，请将此文案投放跑量！");
        }

        for (Map<String, Object> videoSlogenReportMap: generalCopywriteReportList) {

            if(videoSlogenReportMap.get("count") == null){
                videoSlogenReportMap.put("count",0);
            }

            BigDecimal cost = new BigDecimal(Check.isNull(videoSlogenReportMap.get("cost")) ? "0" : videoSlogenReportMap.get("cost").toString());
            BigDecimal click = new BigDecimal(Check.isNull(videoSlogenReportMap.get("click")) ? "0" : videoSlogenReportMap.get("click").toString());
            BigDecimal showNum = new BigDecimal(Check.isNull(videoSlogenReportMap.get("showNum")) ? "0" : videoSlogenReportMap.get("showNum").toString());
            BigDecimal convertNum = new BigDecimal(Check.isNull(videoSlogenReportMap.get("convertNum")) ? "0" : videoSlogenReportMap.get("convertNum").toString());

            //被除数不能为0
            if (StringUtils.equals("0",showNum.toString())){
                videoSlogenReportMap.put("costPerThousandShow", 0);
                videoSlogenReportMap.put("clickRate", 0);
            }else {
                BigDecimal costPerThousandShow = cost.multiply(new BigDecimal(1000)).divide(showNum, 2, BigDecimal.ROUND_HALF_UP);
                videoSlogenReportMap.put("costPerThousandShow", costPerThousandShow);
                BigDecimal clickRate = click.multiply(new BigDecimal(100)).divide(showNum, 2, BigDecimal.ROUND_HALF_UP);
                videoSlogenReportMap.put("clickRate", clickRate);
            }
            if (StringUtils.equals("0", convertNum.toString())) {
                videoSlogenReportMap.put("costConvert", 0);
            }else {
                BigDecimal costConvert = cost.divide(convertNum, 2, BigDecimal.ROUND_HALF_UP);
                videoSlogenReportMap.put("costConvert", costConvert);
            }
            if (StringUtils.equals("0", click.toString())) {
                videoSlogenReportMap.put("costPerClick", 0);
                videoSlogenReportMap.put("convertRate", 0);
            }else {
                BigDecimal costPerClick = cost.divide(click, 2, BigDecimal.ROUND_HALF_UP);
                videoSlogenReportMap.put("costPerClick", costPerClick);
                BigDecimal convertRate = convertNum.multiply(new BigDecimal(100)).divide(click,2,BigDecimal.ROUND_HALF_UP);
                videoSlogenReportMap.put("convertRate", convertRate);
            }
        }

        PageInfo<Map<String, Object>> pageInfo = new PageInfo<Map<String, Object>>(generalCopywriteReportList);
        return Result.successMsg("数据加载成功。",pageInfo);
    }
}
