package org.jeecg.modules.bytedance.advertise.controller;

import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.modules.bytedance.advertise.entity.BytedanceVideoSlogenInfo;
import org.jeecg.modules.bytedance.advertise.service.IBytedanceVideoSlogenInfoService;
import org.jeecg.modules.bytedance.advertise.service.impl.BytedanceVideoSlogenInfoServiceImpl;
import org.jeecg.modules.system.service.ISysRoleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 头条视频文案信息
 * @author jeecg-boot
 * 2021-04-19
 * @version V1.0
 */
@Slf4j
@Api(tags="头条视频文案信息")
@RestController
@RequestMapping("/ctop/bytedanceVideoSlogenInfo")
public class BytedanceVideoSlogenInfoController {
	@Autowired
	private IBytedanceVideoSlogenInfoService bytedanceVideoSlogenInfoService;

	@Autowired
	private ISysRoleService sysRoleService;
	/**
	  * 分页列表查询
	 * @param slogenInfo
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@ApiOperation(value="头条视频文案信息-分页列表查询", notes="头条视频文案信息-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<BytedanceVideoSlogenInfo>> queryPageList(BytedanceVideoSlogenInfo slogenInfo,
																 @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
																 @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
																 HttpServletRequest req) {
		Result<IPage<BytedanceVideoSlogenInfo>> result = new Result<>();
		slogenInfo.setStatus(1);
		QueryWrapper<BytedanceVideoSlogenInfo> queryWrapper = QueryGenerator.initQueryWrapper(slogenInfo, req.getParameterMap());
		Page<BytedanceVideoSlogenInfo> page = new Page<>(pageNo, pageSize);
		IPage<BytedanceVideoSlogenInfo> pageList = bytedanceVideoSlogenInfoService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}

	@ApiOperation(value="指定文案报表数据展示", notes="指定文案报表数据展示")
	@GetMapping(value = "/queryPageListByText")
	public Result queryPageListByText(@RequestParam("accountId") String accountId,
									  @RequestParam(value = "keyWord",defaultValue = "") String keyWord,
									  @RequestParam(value = "startTime",defaultValue = "") String startTime,
									  @RequestParam(value = "endTime",defaultValue = "") String endTime,
									  @RequestParam(value = "pageNo", defaultValue="1") Integer pageNo,
									  @RequestParam(value = "pageSize", defaultValue="10") Integer pageSize
									) {
		try {
			String roleCode = sysRoleService.getRoleCodeByUserId(accountId);
			return bytedanceVideoSlogenInfoService.getVideoSlogenReport(accountId,keyWord, startTime,endTime,pageNo,pageSize);
		}catch (Exception e){
			log.error("指定文案-获取指定文案维度创意报表数据异常",e);
			return Result.error("请求失败，请联系开发人员!");
		}
	}

	/**
	  *   添加
	 * @param slogenInfo
	 * @return
	 */
	@ApiOperation(value="头条视频文案信息-添加", notes="头条视频文案信息-添加")
	@PostMapping(value = "/add")
	public Result<BytedanceVideoSlogenInfo> add(@RequestBody BytedanceVideoSlogenInfo slogenInfo) {
		Result<BytedanceVideoSlogenInfo> result = new Result<>();
		try {
			bytedanceVideoSlogenInfoService.save(slogenInfo);
			result.success("添加成功！");
			return result;
		} catch (Exception e) {
			log.error(e.getMessage(),e);
			result.error500("操作失败");
			return result;
		}
	}

	/**
	 *   添加
	 * @param
	 * @param
	 * @param userId
	 * @return
	 */
	@ApiOperation(value="头条视频文案信息-批量添加", notes="头条视频文案信息-批量添加")
	@PostMapping(value = "/addList")
	public Result<BytedanceVideoSlogenInfo> addList(@RequestParam String[] videoCodeArray,
													@RequestParam JSONArray slogenArray,
													@RequestParam String userId) {
		Result<BytedanceVideoSlogenInfo> result = new Result<>();
		try {
			for (String videoCode: videoCodeArray) {
                bytedanceVideoSlogenInfoService.insertSlogans(videoCode,slogenArray,userId);
			}
			result.success("添加成功！");
			return result;
		} catch (Exception e) {
			log.error(e.getMessage(),e);
			result.error500("操作失败");
			return result;
		}
	}

	/**
	 *  编辑
	 * @param slogenInfo 文案信息
	 * @return
	 */
	@ApiOperation(value="头条视频文案信息-编辑", notes="头条视频文案信息-编辑")
	@PostMapping(value = "/edit")
	public Result<BytedanceVideoSlogenInfo> edit(@RequestBody BytedanceVideoSlogenInfo slogenInfo) {
		Result<BytedanceVideoSlogenInfo> result = new Result<>();
		BytedanceVideoSlogenInfo bytedanceVideoSlogenInfoEntity = bytedanceVideoSlogenInfoService.getById(slogenInfo.getId());
		if(bytedanceVideoSlogenInfoEntity==null) {
			result.error500("未找到对应实体");
			return result;
		}
		LoginUser user = (LoginUser) SecurityUtils.getSubject().getPrincipal();
		slogenInfo.setUpdateBy(user.getId());
		slogenInfo.setUpdateTime(new Date());
		bytedanceVideoSlogenInfoService.updateById(slogenInfo);
		result.success("修改成功!");
		return result;
	}

	/**
	 *   通过id删除
	 * @param id
	 * @return
	 */
	@ApiOperation(value="头条视频文案信息-通过id删除", notes="头条视频文案信息-通过id删除")
	@PostMapping(value = "/delete")
	public Result<?> delete(@RequestParam(name="id") String id) {
		try {
			BytedanceVideoSlogenInfo slogenInfo = bytedanceVideoSlogenInfoService.getById(id);
			if(null == slogenInfo||slogenInfo.getStatus() == 0){
				return Result.ok("此文案不存在或者已经被删除");
			}
			slogenInfo.setStatus(0);
			LoginUser user = (LoginUser) SecurityUtils.getSubject().getPrincipal();
			slogenInfo.setUpdateTime(new Date());
			slogenInfo.setCreateBy(user.getId());
			bytedanceVideoSlogenInfoService.updateById(slogenInfo);
			return Result.ok("删除成功!");
		} catch (Exception e) {
			log.error("删除失败",e.getMessage());
			return Result.error("删除失败!");
		}
	}

	/**
	 * 通过id查询
	 * @param id
	 * @return
	 */
	@ApiOperation(value="头条视频文案信息-通过id查询", notes="头条视频文案信息-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<BytedanceVideoSlogenInfo> queryById(@RequestParam(name="id") String id) {
		Result<BytedanceVideoSlogenInfo> result = new Result<>();
		BytedanceVideoSlogenInfo bytedanceVideoSlogenInfo = bytedanceVideoSlogenInfoService.getById(id);
		if(bytedanceVideoSlogenInfo==null) {
			result.error500("未找到对应实体");
			return result;
		}
		result.setResult(bytedanceVideoSlogenInfo);
		result.setSuccess(true);
		return result;
	}
}
